# Import pytorch packages
import torch
import torch.nn as nn
import torch.optim as optim
import torch.nn.functional as F
import torch.backends.cudnn as cudnn
import torchvision
# import torchvision.transforms as transforms
from torchvision import datasets, transforms, models
from tqdm import tqdm
import numpy as np

import os
import sys
import argparse

CIFAR10_MEAN = (0.4914, 0.4822, 0.4465)
CIFAR10_STD_DEV = (0.2023, 0.1994, 0.2010)


def main():
    batch_size = 128
    device = 'cpu'

    transform = transforms.Compose([
        transforms.ToTensor(),
        transforms.Normalize(CIFAR10_MEAN, CIFAR10_STD_DEV),
    ])

    trainset = torchvision.datasets.CIFAR10(root='./data', train=True,
                                        download=True, transform=transform)
    train_loader = torch.utils.data.DataLoader(trainset, batch_size=batch_size,
                                            shuffle=True, num_workers=2)
    testset = torchvision.datasets.CIFAR10(root='./data', train=False,
                                        download=True, transform=transform)
    test_loader = torch.utils.data.DataLoader(testset, batch_size=batch_size,
                                            shuffle=False, num_workers=2)
    # Retrieve pre-trained model.
    # model = models.resnet18(pretrained=True)
    model = torch.hub.load("chenyaofo/pytorch-cifar-models", "cifar10_vgg16_bn", pretrained=True)
    print("Load model successful")
    # Remove the last layer
    # newClassifier = torch.nn.Sequential(*(list(model.classifier.children())[:-1]))
    # model.classifier = newClassifier
    # newmodel = model
    newmodel = torch.nn.Sequential(*(list(model.children())[:-1]))
    newmodel.to(device)
    newmodel.eval()
    # Initialize result
    dataiter = iter(train_loader)
    images, labels = dataiter.next()
    output = newmodel(images)
    d = output.shape[1]
    print(f"feature size: {d}")
    ntr = len(trainset)
    nte = len(testset)
    print(f"# training: {ntr}, # testing: {nte}" )

    retTrain = np.zeros( (ntr, d) )
    yTrain = np.zeros( ntr )
    retTest = np.zeros( (nte, d) )
    yTest = np.zeros( nte )
    # Start extracting features
    print("Start extraction for training set...")
    for i, (images, target) in enumerate(train_loader):
        images = images.to(device)
        target = target.to(device)
        output = newmodel(images)
        output = output.view( output.shape[0:2] )
        assert( output.shape[1] == d )
        retTrain[ int(i*batch_size):int( (i+1)*batch_size ), : ] = output.detach().numpy()
        yTrain[ int(i*batch_size):int( (i+1)*batch_size ) ] = target.detach().numpy()
    print("Start extraction for testing set...")
    for i, (images, target) in enumerate(test_loader):
        images = images.to(device)
        target = target.to(device)
        output = newmodel(images)
        output = output.view(  output.shape[0:2] )
        assert( output.shape[1] == d )
        retTest[ int(i*batch_size):int( (i+1)*batch_size ), : ] = output.detach().numpy()
        yTest[ int(i*batch_size):int( (i+1)*batch_size ) ] = target.detach().numpy()
    
    np.save( "./data/train_features2.npy", retTrain )    
    np.save( "./data/train_labels2.npy", yTrain )
    np.save( "./data/test_features2.npy", retTest )    
    np.save( "./data/test_labels2.npy", yTest )
    print("Finished!")



if __name__ == "__main__":
    main()
