import torch
import torch.nn as nn
import torch.nn.functional as F

class LinearNN(nn.Module):
    def __init__(self, data_name):
        super().__init__()
        if data_name == "mnist":
            # 1,28x28
            self.fc = nn.Linear(784,10, bias=False)
            # self.fc = nn.Linear(780,10, bias=False)
        else:
            self.fc = nn.Linear(512,10, bias=False)
    def forward(self,x):
        x = torch.flatten(x, 1)
        out = self.fc(x)
        return out

class TwoLayerNN(nn.Module):
    def __init__(self, data_name):
        super().__init__()
        # 1,28x28
        self.relu = nn.ReLU(inplace=True)
        if data_name == "mnist":
            self.fc1 = nn.Linear(784, 256, bias=False)
            # self.fc1 = nn.Linear(780, 256, bias=False)
        else:
            self.fc1 = nn.Linear(512, 256, bias=False)
        self.fc2 = nn.Linear(256, 10, bias=False)
    def forward(self,x):
        x = torch.flatten(x, 1)
        out = self.fc1(x)
        out = self.relu(out)
        out = self.fc2(out)
        return out
    
class SimpleConvNet(nn.Module):
    def __init__(self):
        super().__init__()
        self.conv1 = nn.Conv2d(3, 6, 5, bias=False)
        self.pool = nn.MaxPool2d(2, 2)
        self.conv2 = nn.Conv2d(6, 16, 5, bias=False)
        self.fc1 = nn.Linear(16 * 5 * 5, 128, bias=False)
        self.fc2 = nn.Linear(128, 10, bias=False)
        # self.fc3 = nn.Linear(84, 10, bias=False)

    def forward(self, x):
        x = self.pool(F.relu(self.conv1(x)))
        x = self.pool(F.relu(self.conv2(x)))
        x = torch.flatten(x, 1) # flatten all dimensions except batch
        x = F.relu(self.fc1(x))
        # x = F.relu(self.fc2(x))
        x = self.fc2(x)
        # x = self.fc3(x)
        return x